
include('lib/contrib.js');
include('lib/JavaScriptUtil.js');
include('lib/Parsers.js');
include('lib/InputMask.js');

//////////////////////////////////////////////////////////////////////////////////
//                                                                              
// HMGSCRIPT: Programming For The Web in The Right Way :)
// Roberto Lopez (www.hmgforum.com)
// 
//////////////////////////////////////////////////////////////////////////////////
// 
// I've wrote this library after analize tens of JavaScript samples, code,  
// tutorials and users posts in forums, freely available on the web.
// 
// I've simply attempted to arrange in a simple way, the common knowledge
// available about how to create modal windows and controls on the fly using
// JavaScript.
//
// So, I not consider this code as mine, then, I'm publishing it as public domain.
//
// From the user's perspective, these GUI functions should be as easy to use as
// HMG GUI commands and I've wrote them with such goal in mind.
//
// This library must be considered as pre-alpha.
//
//////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////
//
// This is free and unencumbered software released into the public domain.
// 
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
// 
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// 
// For more information, please refer to <http://unlicense.org/>
//
//////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////
// Library Global Variables
//////////////////////////////////////////////////////////////////////////////////

var cUser = '' ;
var cPassword = '' ;
var nControlCount = 0 ;
var aForms = [];
var nFormsIndex = -1 ;

var TYPES = {
    'undefined'        : 'undefined',
    'number'           : 'number',
    'boolean'          : 'boolean',
    'string'           : 'string',
    '[object Function]': 'function',
    '[object RegExp]'  : 'regexp',
    '[object Array]'   : 'array',
    '[object Date]'    : 'date',
    '[object Error]'   : 'error'
},
TOSTRING = Object.prototype.toString;


//////////////////////////////////////////////////////////////////////////////////
// Form
//////////////////////////////////////////////////////////////////////////////////
   
function Form( cCaption , nWidth , nHeight )
{

	// Create a div for the cover, set its properties and add to the page body.
	// The cover is used since this is a modal window.

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	nFormsIndex++
	aForms [ nFormsIndex ] = cId ;
   	  
	var cover          = document.createElement( "div" )                             ;
	cover.className    = "cover"                                                     ;
	cover.style.width  = document.body.clientWidth                                   ;
	cover.style.height = document.body.clientHeight                                  ;
	cover.id           = 'cover_' + cId                                              ;

	document.body.appendChild( cover )                                               ;

	// Create a div for the window, set its properties and add to the page body.

	var window          = document.createElement( "div" )                            ;
	window.className    = "window"                                                   ;
	window.innerHTML    = "<h1>" + cCaption + "</h1>"                                ;
	window.style.top    = ( document.body.clientHeight / 2 ) - ( nHeight / 2 )       ;
	window.style.left   = ( document.body.clientWidth / 2 ) - ( nWidth / 2 )         ;
	window.style.width  = nWidth                                                     ;
	window.style.height = nHeight                                                    ;
	window.id           = cId                                                        ;

	document.body.appendChild( window );

	this.release = function ()	
	{

		document.getElementById( 'cover_' + aForms [ nFormsIndex ] ).parentNode.removeChild( document.getElementById( 'cover_' + aForms [ nFormsIndex ] ) )

		document.getElementById( aForms [ nFormsIndex ] ).parentNode.removeChild( document.getElementById( aForms [ nFormsIndex ] ) );

		nFormsIndex--;

	}

	this.getId = function ()
	{
		return cId;		
	}
      	                   
}

//////////////////////////////////////////////////////////////////////////////////
// Panel
/////////////////////////////////////////////////////////////////////////////////

function Panel( oParent , nRow , nCol , nWidth , nHeight , cValue )
{

	// Create the panel on the fly, set its properties and append to the specified
	// parent window

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "div" )        ;

	control.className      = "panel"                     ;      
	control.style.position = "absolute"                  ;
	control.style.top      = nRow                        ;
	control.style.left     = nCol                        ;
	control.style.width    = nWidth                      ;
	control.style.height   = nHeight                     ;
	control.id             = cId                         ;
	control.innerHTML      = cValue                      ;
	control.style.border   = "solid 1px rgb(0,0,255)"    ;
	control.style.overflow = "scroll"                    ;
    
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getId = function ()
	{
		return cId;		
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// Button 
/////////////////////////////////////////////////////////////////////////////////

function Button( oParent , nRow , nCol , cCaption , cOnClick , nWidth )
{

	// Create the button on the fly, set its properties and append to the specified
	// parent window

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "input" )      ;
      
	control.type           = "button"                    ;
	control.value          = cCaption                    ;
	control.onclick        = Function( cOnClick )        ;
	control.style.position = "absolute"                  ;
	control.style.top      = nRow                        ;
	control.style.left     = nCol                        ;
	control.style.height   = 35                          ;
	control.id             = cId                         ;

	if ( type(nWidth) == "number" )
	{
		control.style.width    = nWidth               ;
	}
	else
	{
		control.style.width    = 120                 ;
	}
    
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.setValue = function ( cValue )	
	{
		control.value = cValue ;		
	}

	this.getValue = function ( cValue )	
	{
		return control.value ;		
	}

	this.getId = function ()
	{
		return cId;		
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// ComboBox
//////////////////////////////////////////////////////////////////////////////////

function ComboBox( oParent , nRow , nCol , aText , aValues , nSelectedIndex )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "select" );

	control.style.position = "absolute" ;      
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.id             = cId        ;

	for ( var i = 0 ; i < aText.length ; i++ )
	{
		control.options[i] = new Option( aText[i] , aValues[i] );
	}

	if ( type(nSelectedIndex) == "number" )
	{
		control.selectedIndex = nSelectedIndex ;
	}

	this.getValue = function( nIndex )
	{
		return control.options[ nIndex ].value;
	}

	this.setValue = function( nIndex , cValue )
	{
		options[ nIndex ].value = cValue ;
	}

	this.setSelectedIndex = function( nIndex )
	{
		control.selectedIndex = nIndex ;
	}

	this.getSelectedIndex = function()
	{
		return control.selectedIndex ;
	}

	this.getId = function ()
	{
		return cId;		
	}

	document.getElementById( oParent.getId() ).appendChild( control )                ;

}   

//////////////////////////////////////////////////////////////////////////////////
// Label 
//////////////////////////////////////////////////////////////////////////////////

function Label( oParent , nRow , nCol , cValue )
{

	// Create the label on the fly, set its properties and append to the specified
	// parent window

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "span" );
      
	control.className      = "label"    ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.innerHTML      = cValue     ;
	control.id             = cId        ;
      
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getId = function ()
	{
		return cId;		
	}

	this.getValue = function()
	{
		return document.getElementById(cId).innerHTML;
	}

	this.setValue = function( cValue )
	{
		document.getElementById(cId).innerHTML = cValue ;
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// TextBox
//////////////////////////////////////////////////////////////////////////////////

function TextBox( oParent , nRow , nCol , cValue , bPassword )
{

	var numParser;
	var numMask;

	var cDataType = 'string';

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "input" );

	if ( bPassword )
	{
		control.type           = "password"     ;
	}
	else
	{
		control.type           = "text"     ;
	}

	control.className      = "textbox"  ;
	control.style.position = "absolute" ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.value          = cValue     ;
	control.id             = cId        ;
	control.name           = cId        ;
     
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getAsNumber = function( )
	{
		if ( cDataType == 'number')
		{
			return numMask.getAsNumber();
		}
	}

	this.setAsNumber = function( nValue )
	{
		if ( cDataType == 'number')
		{
			numMask.setAsNumber( nValue );
		}
	}

	this.getValue = function()
	{
		return document.getElementById(cId).value;
	}

	this.setValue = function( cValue )
	{
		document.getElementById(cId).value = cValue ;
	}

	this.getId = function ()
	{
		return cId;		
	}

	this.setFocus = function ()
	{
		control.focus();
	}

	this.disable = function()
	{
		document.getElementById( this.getId() ).disabled = true;
	}

	this.enable = function()
	{
		document.getElementById( this.getId() ).disabled = false;
	}

	this.setNumericMask = function( nIintDigits , nDecimals , cDecimalSeparator , cGgroupSeparator , bAllowNegative, bLeftToRight, cCurrencySymbol )
	{

		cDataType = 'number';

		control.style.textAlign      = "right"    ;

	        numParser = new NumberParser( nDecimals , 
				cDecimalSeparator , cGgroupSeparator , true);

	
		if ( cGgroupSeparator.length == 0 )
		{
	        	numParser.useGrouping = false;
		}
		else
		{
	        	numParser.useGrouping = true;
		}
	
		if ( cCurrencySymbol.length == 0 )
		{
	        	numParser.useCurrency = false;
		}
		else
		{
        		numParser.useCurrency = true;
		        numParser.currencySymbol = cCurrencySymbol
		        numParser.negativeParenthesis = true;
		        numParser.currencyInside = true;
		}

       		numMask = new NumberMask(numParser , cId, nIintDigits );
		numMask.allowNegative = bAllowNegative ;
		numMask.leftToRight = bLeftToRight ;

	}

	this.setStringMask = function( cMask )
	{
		cDataType = 'string';
	        var Mask = new InputMask( cMask , cId );
	}

	this.isValid = function ()
	{
		return getObject( this.getId() ).mask.isComplete();
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// CheckBox
//////////////////////////////////////////////////////////////////////////////////

function CheckBox( oParent , nRow , nCol , bChecked )
{

	// Create the CheckBox on the fly, set its properties and append to the specified
	// parent window

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "input" );
   
	control.type           = "checkbox"     ;
	control.style.position = "absolute" ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.id             = cId        ;

	if ( type(bChecked) == 'boolean' )
	{
		control.checked        = bChecked   ;
	}
	else if ( type(bChecked) == 'string' )
	{
		if ( bChecked == 'Yes' )
		{
			control.checked = true   ;
		}
		else
		{
			control.checked = false   ;
		}

	}
      
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.check = function()
	{
		document.getElementById(cId).checked = true;
	}

	this.unCheck = function()
	{
		document.getElementById(cId).checked = false ;
	}

	this.isChecked = function()
	{

		if ( document.getElementById(cId).checked == true )
		{
			return true;	
		}
		else
		{
			return false;
		}

	}

	this.getValue = function()
	{
		if ( document.getElementById(cId).checked == true )
		{
			return 'Yes';
		}
		else
		{
			return 'No';
		}
	}

	this.setValue = function( cValue )
	{
		if ( cValue == 'Yes' )
		{
			document.getElementById(cId).checked == true ;
		}
		else if ( cValue == 'No' )
		{
			document.getElementById(cId).checked == false ;
		}

	}

	this.getId = function ()
	{
		return cId;		
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// Image
//////////////////////////////////////////////////////////////////////////////////

function Image( oParent , nRow , nCol , cSrc )
{

	// Create the Image on the fly, set its properties and append to the specified
	// parent window

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "img" )                          ;            
      
	control.style.position = "absolute"                                      ;
	control.className      = "image"                                         ;
	control.id             = cId                                             ;
	control.style.top      = nRow                                            ;
	control.style.left     = nCol                                            ;
	control.src            = cSrc                                            ;
      
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getId = function ()
	{
		return cId;		
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// Browse
//////////////////////////////////////////////////////////////////////////////////

function Browse( oParent , nRow , nCol , nWidth , nHeight , cTable , aColumns , aHeaders , cForExpr , cOrder , nInnerWidth , nPageSize )
{

	if ( type(nPageSize) == 'undefined' )
	{
		nPageNumber = 0 ;
		nPageSize = 0 ;
	}
	else
	{
		nPageNumber = 1;
	}

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var query = 'cId=' + cId + '&' + 'cDbf=' + cTable + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'aHeaders=' + escape(JsToHrb( aHeaders )) + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cOrder=' + escape(cOrder) + '&' + 'nInnerWidth=' + nInnerWidth + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword + '&' + 'nPageSize=' + nPageSize + '&' + 'nPageNumber=' + nPageNumber ;

	var control = document.createElement( "div" )        ;

	var panel = cId + '_panel';

	control.className      = "browse"                    ;      
	control.style.position = "absolute"                  ;
	control.style.top      = nRow                        ;
	control.style.left     = nCol                        ;
	control.style.width    = nWidth                      ;
	control.style.height   = nHeight                     ;
	control.id             = panel                       ;
	control.innerHTML      = "Loading..."                ;
	control.style.border   = "solid 1px rgb(200,200,200)"    ;
	control.style.overflow = "scroll"                    ;
    
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	xmlHttp=new XMLHttpRequest();
	xmlHttp.open( "POST" , "/cgi-bin/browse.hrb"  , false ) ;
	xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlHttp.send( query );
	document.getElementById(panel).innerHTML = xmlHttp.responseText;

	this.getPageSize = function()
	{
		return nPageSize ;
	}

	this.getSelectedRowCount = function()
	{
		var i, n ;

		n = 0 ;

		for ( i=1 ; i < document.getElementById(cId).rows.length ; i++ )
		{
			if ( document.getElementById(cId).rows[i].cells[0].getElementsByTagName("input")[0].checked == true ) 
			{
				n++ ;
			}			 
		}
		return n;
	};

	this.getSelectedRows = function()
	{
		var selectedRows=new Array();
		var i, code, first, last, n ;

		n = 0 ;

		for ( i=1 ; i < document.getElementById(cId).rows.length ; i++ )
		{
			if ( document.getElementById(cId).rows[i].cells[0].getElementsByTagName("input")[0].checked == true ) 
			{
				selectedRows[n] = i ;
				n++ ;
			}			 
		}
		return selectedRows;
	};

	this.getRowCount = function()
	{
		return document.getElementById(cId).rows.length;
	}

	this.getCell = function( nRow , nCOl )
	{
		return trim(document.getElementById(cId).rows[nRow].cells[nCOl].childNodes[0].data);
	}

	this.select = function( nRow )
	{
		document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0].checked = true ;
		changeState( document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0] );
	}

	this.unSelect = function( nRow )
	{
		document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0].checked = false ;
		changeState( document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0] );
	}

	this.refresh = function(  )
	{ 

		xmlHttp=new XMLHttpRequest();
		xmlHttp.open( "POST" , "/cgi-bin/browse.hrb"  , false ) ;
		xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlHttp.send( query );
		document.getElementById(panel).innerHTML = xmlHttp.responseText;
	}

	this.loadPage = function( nPage )
	{ 

		query = 'cId=' + cId + '&' + 'cDbf=' + cTable + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'aHeaders=' + escape(JsToHrb( aHeaders )) + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cOrder=' + escape(cOrder) + '&' + 'nInnerWidth=' + nInnerWidth + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword + '&' + 'nPageSize=' + nPageSize + '&' + 'nPageNumber=' + nPage ;

		xmlHttp=new XMLHttpRequest();
		xmlHttp.open( "POST" , "/cgi-bin/browse.hrb"  , false ) ;
		xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlHttp.send( query );
		document.getElementById(panel).innerHTML = xmlHttp.responseText;

	}

	this.getRecordCount = function()
	{ 

		query = 'cDbf=' + cTable + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cOrder=' + escape(cOrder) + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword ;

		xmlHttp=new XMLHttpRequest();
		xmlHttp.open( "POST" , "/cgi-bin/getrecordcount.hrb"  , false ) ;
		xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlHttp.send( query );
		return xmlHttp.responseText;
	}

	this.getRowCount = function()
	{
		return document.getElementById(cId).rows.length;
	}

	this.getPageSize = function()
	{
		return nPageSize;
	}

	this.getId = function ()
	{
		return cId;		
	}

	this.setOrder = function( cValue )
	{ 

		cOrder = cValue ;

		query = 'cId=' + cId + '&' + 'cDbf=' + cTable + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'aHeaders=' + escape(JsToHrb( aHeaders )) + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cOrder=' + escape(cOrder) + '&' + 'nInnerWidth=' + nInnerWidth + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword + '&' + 'nPageSize=' + nPageSize + '&' + 'nPageNumber=' + '1' ;

		xmlHttp=new XMLHttpRequest();
		xmlHttp.open( "POST" , "/cgi-bin/browse.hrb"  , false ) ;
		xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlHttp.send( query );
		document.getElementById(panel).innerHTML = xmlHttp.responseText;
	}


	this.setFilter = function( cValue )
	{ 

		cForExpr = cValue ;

		query = 'cId=' + cId + '&' + 'cDbf=' + cTable + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'aHeaders=' + escape(JsToHrb( aHeaders )) + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cOrder=' + escape(cOrder) + '&' + 'nInnerWidth=' + nInnerWidth + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword + '&' + 'nPageSize=' + nPageSize + '&' + 'nPageNumber=' + '1' ;

		xmlHttp=new XMLHttpRequest();
		xmlHttp.open( "POST" , "/cgi-bin/browse.hrb"  , false ) ;
		xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		xmlHttp.send( query );
		document.getElementById(panel).innerHTML = xmlHttp.responseText;
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// EditBox
//////////////////////////////////////////////////////////////////////////////////

function EditBox( oParent , nRow , nCol , cValue )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "textarea" );

	control.id             = cId        ;
	control.style.position = "absolute" ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.style.width    = 300        ;
	control.style.height   = 150        ;
	control.value          = cValue     ;

	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getValue = function()
	{
		return document.getElementById(cId).value;
	}

	this.setValue = function( cValue )
	{
		document.getElementById(cId).value = cValue ;
	}

	this.getId = function ()
	{
		return cId;		
	}
      
}   

//////////////////////////////////////////////////////////////////////////////////
// Delete
//////////////////////////////////////////////////////////////////////////////////

function Delete ( cDbf , cForExpr )
{

	var cmd = 'cDbf=' + cDbf + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword ;

	var xmlHttp=new XMLHttpRequest();
	xmlHttp.open( "POST" , "/cgi-bin/delete.hrb" , false ) ;
	xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlHttp.send( cmd );

	if ( xmlHttp.responseText == '0' )
	{
		alert('DELETE: No Records Found!');
	}

}

//////////////////////////////////////////////////////////////////////////////////
// Query
//////////////////////////////////////////////////////////////////////////////////

function Query ( cDbf, aColumns, cForExpr, cOrder )
{

	var query = 'cDbf=' + cDbf + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cOrder=' + escape(cOrder) + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword ;

	var xmlHttp=new XMLHttpRequest();
	xmlHttp.open( "POST" , "/cgi-bin/query.hrb" , false ) ;
	xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlHttp.send( query );

	var tmp = xmlHttp.responseText;

	if ( tmp.length == 0 ) 
	{
		return '';
	}

        var strResponseArray = tmp.split('\n');

	var strContentArray = strResponseArray[0].split('\t');
        
	var nRecordCount = strResponseArray.length;

	var nFieldCount = strContentArray.length;

	var aRecordSet = new Array(nRecordCount); 

	for ( var i = 0; i < nRecordCount; i++) 
	{ 

		strContentArray = strResponseArray[i].split('\t');

		aRecordSet[i] =  new Array(nFieldCount);  

		for (j=0; j < nFieldCount; j++) 
		{ 
			aRecordSet[i][j] = strContentArray[j]; 
		} 

	} 

	return aRecordSet ;

}


//////////////////////////////////////////////////////////////////////////////////
// Append
//////////////////////////////////////////////////////////////////////////////////

function Append ( cDbf , aColumns , aValues )
{

	var cmd = 'cDbf=' + cDbf + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'aValues=' + escape(JsToHrb( aValues ) ) + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword ;

	var xmlHttp=new XMLHttpRequest();
	xmlHttp.open( "POST" , "/cgi-bin/append.hrb"  , false ) ;
	xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlHttp.send( cmd );

	if ( xmlHttp.responseText != 'OK' )
	{
		alert( xmlHttp.responseText );
	}

}

//////////////////////////////////////////////////////////////////////////////////
// Modify
//////////////////////////////////////////////////////////////////////////////////

function Modify ( cDbf , aColumns , aValues , cForExpr )
{

	var cmd = 'cDbf=' + cDbf + '&' + 'aColumns=' + escape(JsToHrb( aColumns )) + '&' + 'aValues=' + escape(JsToHrb( aValues ) ) + '&' + 'cForExpr=' + escape(cForExpr) + '&' + 'cUser=' + cUser + '&' + 'cPassword=' + cPassword ;

	var xmlHttp=new XMLHttpRequest();
	xmlHttp.open( "POST" , "/cgi-bin/modify.hrb"  , false ) ;
	xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlHttp.send( cmd );

	if ( xmlHttp.responseText == '0' )
	{
		alert('MODIFY: No Records Found!');
	}

}

//////////////////////////////////////////////////////////////////////////////////
// Login
//////////////////////////////////////////////////////////////////////////////////

function Login ( cUsr , cPasw )
{

	cUser = cUsr ;
	cPassword = cPasw ;

	var cmd = 'cUser=' + cUsr + '&' + 'cPassword=' + cPasw ;

	var xmlHttp=new XMLHttpRequest();
	xmlHttp.open( "POST" , "/cgi-bin/login.hrb" , false ) ;
	xmlHttp.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
	xmlHttp.send( cmd );

	return xmlHttp.responseText;
}

//////////////////////////////////////////////////////////////////////////////////
// Logout
//////////////////////////////////////////////////////////////////////////////////

function Logout()
{
	cUser = '' ;
	cPassword = '' ;
}

//---------------------------------------------------------------------------------------------//
// JsToHrb (Internal Function) - Converts a JS array to a Harbour one 
//---------------------------------------------------------------------------------------------//

function JsToHrb( aArray )
{

	var c ;
	var i ;
	var cTempColumns ;
	cTempColumns = ' { ' ;

	for ( var i = 0 ; i < aArray.length ; i++ )
	{

		if ( type( aArray[i] ) == 'boolean' )
		{
			if ( aArray [i] == true )
			{
				cTempColumns = cTempColumns + '"Yes"' ;
			}
			else
			{
				cTempColumns = cTempColumns + '"No"' ;
			}
		}
		else
		{
			cTempColumns = cTempColumns + '"' + aArray [i] + '"' ;
		}

		if ( i != aArray.length - 1 )
		{
			cTempColumns = cTempColumns + ' , ' ;
		}	

	}

	cTempColumns = cTempColumns + ' } ' ;

	return cTempColumns;

}

//---------------------------------------------------------------------------------------------//
// dateToYMD (Internal Function) - Converts JS date to a Harbour date constant
//---------------------------------------------------------------------------------------------//

function dateToYMD(date)
{
    var d = date.getDate();
    var m = date.getMonth()+1;
    var y = date.getFullYear();
    return '' + y +'-'+ (m<=9?'0'+m:m) +'-'+ (d<=9?'0'+d:d);
}

//---------------------------------------------------------------------------------------------//
// Type (Internal Function) - Returns the type od a JS variable
//---------------------------------------------------------------------------------------------//

function type(o) {
    return TYPES[typeof o] || TYPES[TOSTRING.call(o)] || (o ? 'object' : 'null');
};


//////////////////////////////////////////////////////////////////////////////////
// Grid
//////////////////////////////////////////////////////////////////////////////////

function Grid( oParent, nRow, nCol, nWidth, nHeight, aHeaders, nInnerWidth )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	// Create a scrollable div as container for the table.

	var control = document.createElement( "div" )        ;

	panel = cId + '_panel';

	control.style.position = "absolute"                  ;
	control.style.top      = nRow                        ;
	control.style.left     = nCol                        ;
	control.style.width    = nWidth                      ;
	control.style.height   = nHeight                     ;
	control.id             = panel                       ;
	control.style.border   = "solid 1px rgb(200,200,200)"    ;
	control.style.overflow = "scroll"                    ;
    
	document.getElementById( oParent.getId() ).appendChild( control )                ;

        // creates a <table> element and a <tbody> element

        var tbl     = document.createElement("table");
        var tblBody = document.createElement("tbody");

	tbl.className = "grid";

	// Headers

	var row = document.createElement("tr");

	var cell = document.createElement("th");
	var cellText = document.createTextNode( '#' );
	cell.appendChild(cellText);
	row.appendChild(cell);

	for( var n = 0; n < aHeaders.length; n++ )
	{	
		var cell = document.createElement("th");
		var cellText = document.createTextNode( aHeaders [n] );
		cell.appendChild(cellText);
		row.appendChild(cell);
	}

	tblBody.appendChild(row);

	// End Headers

        tbl.setAttribute("border", "0");
        tbl.setAttribute("width", nInnerWidth );
        tbl.setAttribute("id", cId );

        tbl.appendChild(tblBody);

	control.appendChild( tbl )                ;

	// addRow Method ///////////////////////////////////////////////////////////////////////

	this.addRow = function( aRow )
	{
		var row = document.createElement("tr");

		var cell = document.createElement("td");
		var chkbox = document.createElement('input'); chkbox.type = "checkbox";
		chkbox.onclick = function() { changeState( this ); };
		cell.appendChild(chkbox)
		row.appendChild(cell);

		for( var n = 0; n < aRow.length; n++ )
		{	
			var cell = document.createElement("td");
			var cellText = document.createTextNode( aRow [n] );
			cell.appendChild(cellText);
			row.appendChild(cell);
		}

		tblBody.appendChild(row);
	}



	// getRowCount Method //////////////////////////////////////////////////////////////////

	this.getRowCount = function()
	{
		return document.getElementById(cId).rows.length;
	}

	// getCell Method //////////////////////////////////////////////////////////////////////

	this.getCell = function( nRow , nCOl )
	{
		return document.getElementById(cId).rows[nRow].cells[nCOl].childNodes[0].data;
	}

	// setCell Method //////////////////////////////////////////////////////////////////////

	this.setCell = function( nRow , nCOl , cValue )
	{
		document.getElementById(cId).rows[nRow].cells[nCOl].childNodes[0].data = cValue ;
	}

	// select Method ///////////////////////////////////////////////////////////////////////

	this.select = function( nRow )
	{
		document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0].checked = true ;
		changeState( document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0] );
	}

	// unSelect Method /////////////////////////////////////////////////////////////////////

	this.unSelect = function( nRow )
	{
		document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0].checked = false ;
		changeState( document.getElementById(cId).rows[nRow].cells[0].getElementsByTagName("input")[0] );
	}

	// deleteRow ///////////////////////////////////////////////////////////////////////////

	this.deleteRow = function( nRow )
	{
		document.getElementById( cId ).deleteRow(nRow);
	}

	// getId Method ////////////////////////////////////////////////////////////////////////

	this.getId = function ()
	{
		return cId;		
	}

	// getSelectedRows /////////////////////////////////////////////////////////////////////

	this.getSelectedRows = function()
	{
		var selectedRows=new Array();
		var i, code, first, last, n ;

		n = 0 ;

		for ( i=1 ; i < document.getElementById(cId).rows.length ; i++ )
		{
			if ( document.getElementById(cId).rows[i].cells[0].getElementsByTagName("input")[0].checked == true ) 
			{
				selectedRows[n] = i ;
				n++ ;
			}			 
		}
		return selectedRows;
	};

	// getSelectedRowCount Method //////////////////////////////////////////////////////////

	this.getSelectedRowCount = function()
	{
		var i, n ;

		n = 0 ;

		for ( i=1 ; i < document.getElementById(cId).rows.length ; i++ )
		{
			if ( document.getElementById(cId).rows[i].cells[0].getElementsByTagName("input")[0].checked == true ) 
			{
				n++ ;
			}			 
		}
		return n;
	};

}   

//////////////////////////////////////////////////////////////////////////////////
// RadioGroup
//////////////////////////////////////////////////////////////////////////////////

function RadioGroup( oParent , nRow , nCol , aOptions , nSelectedIndex )
{

	var n ;

	var aId = new Array();

	var cName = 'control' + (nControlCount).toString() ; nControlCount++ ;

	for( n = 0; n < aOptions.length; n++ )
	{	

		// Create Radio Button

		var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;
		var control = document.createElement( "input" );

		aId [n] = cId;

		control.type           = "radio"    ;
		control.name           = cName      ;
		control.id             = cId        ;
		control.style.position = "absolute" ;
		control.style.top      = nRow       ;
		control.style.left     = nCol       ;

		document.getElementById( oParent.getId() ).appendChild( control )                ;
	
		// Create the label for the button

		var control = document.createElement( "div" );
		control.id             = cId + '_label' ;
		control.className      = "label"        ;
		control.style.position = "absolute"     ;
		control.style.top      = nRow           ;
		control.style.left     = nCol + 20      ;
		control.innerHTML      = aOptions[n]    ;

		document.getElementById( oParent.getId() ).appendChild( control )                ;

		nRow+= 20;

	}

	// Set initial value

	document.getElementById( aId [nSelectedIndex] ).checked = true ;

	this.getSelectedIndex = function()
	{
		for( n = 0; n < aId.length; n++ )
		{	
			if ( document.getElementById( aId [n] ).checked == true )
			{
				return n;
			}
		}
	}

	this.setSelectedIndex = function( nIndex )
	{
		document.getElementById( aId [nIndex] ).checked = true ;
	}

	this.getId = function ( nIndex )
	{
		return aId [nIndex] ;		
	}


}   

//---------------------------------------------------------------------------------------------//
// changeState (internal for Grid and Browse use)
//---------------------------------------------------------------------------------------------//

function changeState( oCheck )
{
	if( oCheck.checked )
	{
		oCheck.parentNode.parentNode.className = "selected";
	}
	else
	{
		oCheck.parentNode.parentNode.className = "unSelected";
	}
}   

//////////////////////////////////////////////////////////////////////////////////
// Spinner
//////////////////////////////////////////////////////////////////////////////////

function Spinner( oParent , nRow , nCol , nValue )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "input" );
   
	control.type           = "number"   ;
	control.className      = "spinner";
	control.style.position = "absolute" ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.value          = nValue     ;
	control.id             = cId        ;
      
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getValue = function()
	{
		return document.getElementById(cId).value;
	}

	this.setValue = function( cValue )
	{
		document.getElementById(cId).value = cValue ;
	}

	this.getId = function ()
	{
		return cId;		
	}

	this.setMin = function( nMin )
	{
		control.min = nMin ;
	}

	this.setMax = function( nMax )
	{
		control.max = nMax ;
	}

	this.setStep = function( nStep )
	{
		control.step = nStep ;
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// Sliider
//////////////////////////////////////////////////////////////////////////////////

function Slider( oParent , nRow , nCol , nValue )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "input" );
   
	control.type           = "range"    ;
	control.className      = "spinner"  ;
	control.style.position = "absolute" ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.value          = nValue     ;
	control.id             = cId        ;
      
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getValue = function()
	{
		return document.getElementById(cId).value;
	}

	this.setValue = function( cValue )
	{
		document.getElementById(cId).value = cValue ;
	}

	this.getId = function ()
	{
		return cId;		
	}

	this.setMin = function( nMin )
	{
		control.min = nMin ;
	}

	this.setMax = function( nMax )
	{
		control.max = nMax ;
	}

	this.setStep = function( nStep )
	{
		control.step = nStep ;
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// DatePicker
//////////////////////////////////////////////////////////////////////////////////

function DatePicker( oParent , nRow , nCol , dValue )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "input" );
   
	control.type           = "date"    ;
	control.className      = "spinner"  ;
	control.style.position = "absolute" ;
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.value          = dValue     ;
	control.id             = cId        ;
      
	document.getElementById( oParent.getId() ).appendChild( control )                ;

	this.getValue = function()
	{
		return document.getElementById(cId).value;
	}

	this.setValue = function( cValue )
	{
		document.getElementById(cId).value = cValue ;
	}

}   

//////////////////////////////////////////////////////////////////////////////////
// LISTBOX
//////////////////////////////////////////////////////////////////////////////////

function ListBox( oParent , nRow , nCol , aText , aValues , nSize , nSelectedIndex )
{

	var cId = 'control' + (nControlCount).toString() ; nControlCount++ ;

	var control = document.createElement( "select" );

	control.style.position = "absolute" ;      
	control.style.top      = nRow       ;
	control.style.left     = nCol       ;
	control.id             = cId        ;
	control.size           = nSize      ;

	for ( i = 0 ; i < aText.length ; i++ )
	{
		control.options[i] = new Option( aText[i] , aValues[i] );
	}

	if ( type(nSelectedIndex) == "number" )
	{
		control.selectedIndex = nSelectedIndex ;
	}

	this.getValue = function( nIndex )
	{
		return control.options[ nIndex ].value;
	}

	this.setValue = function( nIndex , cValue )
	{
		options[ nIndex ].value = cValue ;
	}

	this.setSelectedIndex = function( nIndex )
	{
		control.selectedIndex = nIndex ;
	}

	this.getSelectedIndex = function()
	{
		return control.selectedIndex ;
	}

	this.getId = function ()
	{
		return cId;		
	}

	document.getElementById( oParent.getId() ).appendChild( control )                ;

}   

//////////////////////////////////////////////////////////////////////////////////
// TRIM
//////////////////////////////////////////////////////////////////////////////////

function trim(s)
{
	return rtrim(ltrim(s));
}

//////////////////////////////////////////////////////////////////////////////////
// LTRIM
//////////////////////////////////////////////////////////////////////////////////

function ltrim(s)
{
	var l=0;
	while(l < s.length && s[l] == ' ')
	{	l++; }
	return s.substring(l, s.length);
}

//////////////////////////////////////////////////////////////////////////////////
// RTRIM
//////////////////////////////////////////////////////////////////////////////////

function rtrim(s)
{
	var r=s.length -1;
	while(r > 0 && s[r] == ' ')
	{	r-=1;	}
	return s.substring(0, r+1);
}


function include(file)
{
 
  var script  = document.createElement('script');
  script.src  = file;
  script.type = 'text/javascript';
  script.defer = true;
 
  document.getElementsByTagName('head').item(0).appendChild(script);
 
}